@@ -7,6 +7,7 @@ import io
 import os
 import tempfile
 from pathlib import Path
+import time
 
 class SharePointBytesIO(io.BytesIO):
     """Class to read, move, list files on Sharepoint, including Teams files folder.
@@ -59,6 +60,7 @@ class SharePointBytesIO(io.BytesIO):
     def list_files(self, folder, site=None, keep_only=None, start_with=None, str_contains=None):
         """List all files in a folder
         """
+
         if site:
             self.ctx = self._connect(site)
         
@@ -191,27 +193,40 @@ class SharePointBytesIO(io.BytesIO):
         ctx.execute_query()
 
 
-    def download(self, file, full_URL=False, folder=None, site=None, download_path=None, get_download_path=False):
+    def download(self, file, download_path=None, get_download_path=False):
         """Download file in a temporary directory
         """
-        if folder:
-            self.folder = folder
 
-        if site:
-            self.ctx = self._connect(site)
-
-    
         ctx = self.ctx
-        if full_URL:
-            file_url = file
-        else:
-            file_url = self.site + self.folder + '/' + file
+        site = self.site
+
+        file_url = file
 
         if download_path == None:
             download_path = os.path.join(tempfile.mkdtemp(), os.path.basename(file_url))
-            with open(download_path, "wb") as local_file:
-                file = ctx.web.get_file_by_server_relative_url(file_url).download(local_file).execute_query()
-            print("[Ok] file has been downloaded: {0}".format(download_path))
+        else:
+            download_path = os.path.join(download_path, os.path.basename(file_url))
+
+        success_download = False
+        sleeping_time = 1
+
+        while not success_download:
+            try:
+                with open(download_path, "wb") as local_file:
+                    file = ctx.web.get_file_by_server_relative_url(file_url).download(local_file).execute_query()
+                success_download = True
+
+            except Exception as e:
+                print(e) #HTTPSConnectionPool
+                print("Failing download, try again in " + str(sleeping_time) + " seconds")
+                time.sleep(sleeping_time)
+
+            if sleeping_time > 65:
+                raise Exception("Sleeping time exceed 128 seconds.")
+
+            sleeping_time = sleeping_time * 2
+
+        print("[Ok] file has been downloaded: {0}".format(download_path))
 
         if get_download_path:
             return download_path
\ No newline at end of file
